@@ -17,9 +17,11 @@
  * with this program; if not, write to the Free Software Foundation, Inc.,
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
- * (C) Copyright 2007 - 2009 Red Hat, Inc.
+ * (C) Copyright 2007 - 2010 Red Hat, Inc.
  */
 
+#include "config.h"
+
 #include <string.h>
 
 #include <glib.h>
@@ -133,38 +135,69 @@ wireless_security_unref (WirelessSecurity *sec)
 	g_return_if_fail (sec != NULL);
 	g_return_if_fail (sec->refcount > 0);
 
-	g_assert (sec->destroy);
-
 	sec->refcount--;
 	if (sec->refcount == 0) {
-		g_object_unref (sec->xml);
-		g_object_unref (sec->ui_widget);
-		(*(sec->destroy)) (sec);
+		if (sec->destroy)
+			sec->destroy (sec);
+
+		if (sec->builder)
+			g_object_unref (sec->builder);
+		if (sec->ui_widget)
+			g_object_unref (sec->ui_widget);
+		g_slice_free1 (sec->obj_size, sec);
 	}
 }
 
-void
-wireless_security_init (WirelessSecurity *sec,
+WirelessSecurity *
+wireless_security_init (gsize obj_size,
                         WSValidateFunc validate,
                         WSAddToSizeGroupFunc add_to_size_group,
                         WSFillConnectionFunc fill_connection,
                         WSUpdateSecretsFunc update_secrets,
                         WSDestroyFunc destroy,
-                        GladeXML *xml,
-                        GtkWidget *ui_widget,
+                        const char *ui_file,
+                        const char *ui_widget_name,
                         const char *default_field)
 {
+	WirelessSecurity *sec;
+	GError *error = NULL;
+
+	g_return_val_if_fail (obj_size > 0, NULL);
+	g_return_val_if_fail (ui_file != NULL, NULL);
+	g_return_val_if_fail (ui_widget_name != NULL, NULL);
+
+	sec = g_slice_alloc0 (obj_size);
+	g_assert (sec);
+
 	sec->refcount = 1;
+	sec->obj_size = obj_size;
 
 	sec->validate = validate;
 	sec->add_to_size_group = add_to_size_group;
 	sec->fill_connection = fill_connection;
 	sec->update_secrets = update_secrets;
 	sec->destroy = destroy;
-
-	sec->xml = xml;
-	sec->ui_widget = ui_widget;
 	sec->default_field = default_field;
+
+	sec->builder = gtk_builder_new ();
+	if (!gtk_builder_add_from_file (sec->builder, ui_file, &error)) {
+		g_warning ("Couldn't load UI builder file %s: %s",
+		           ui_file, error->message);
+		g_error_free (error);
+		wireless_security_unref (sec);
+		return NULL;
+	}
+
+	sec->ui_widget = GTK_WIDGET (gtk_builder_get_object (sec->builder, ui_widget_name));
+	if (!sec->ui_widget) {
+		g_warning ("Couldn't load UI widget '%s' from UI file %s",
+		           ui_widget_name, ui_file);
+		wireless_security_unref (sec);
+		return NULL;
+	}
+	g_object_ref_sink (sec->ui_widget);
+
+	return sec;
 }
 
 GtkWidget *
@@ -204,11 +237,11 @@ ws_802_1x_add_to_size_group (WirelessSecurity *sec,
 	GtkTreeIter iter;
 	EAPMethod *eap;
 
-	widget = glade_xml_get_widget (sec->xml, label_name);
+	widget = GTK_WIDGET (gtk_builder_get_object (sec->builder, label_name));
 	g_assert (widget);
 	gtk_size_group_add_widget (size_group, widget);
 
-	widget = glade_xml_get_widget (sec->xml, combo_name);
+	widget = GTK_WIDGET (gtk_builder_get_object (sec->builder, combo_name));
 	g_assert (widget);
 
 	model = gtk_combo_box_get_model (GTK_COMBO_BOX (widget));
@@ -228,7 +261,7 @@ ws_802_1x_validate (WirelessSecurity *sec, const char *combo_name)
 	EAPMethod *eap = NULL;
 	gboolean valid = FALSE;
 
-	widget = glade_xml_get_widget (sec->xml, combo_name);
+	widget = GTK_WIDGET (gtk_builder_get_object (sec->builder, combo_name));
 	g_assert (widget);
 
 	model = gtk_combo_box_get_model (GTK_COMBO_BOX (widget));
@@ -254,7 +287,7 @@ ws_802_1x_auth_combo_changed (GtkWidget *combo,
 	GtkWidget *eap_widget;
 	GtkWidget *eap_default_widget = NULL;
 
-	vbox = glade_xml_get_widget (sec->xml, vbox_name);
+	vbox = GTK_WIDGET (gtk_builder_get_object (sec->builder, vbox_name));
 	g_assert (vbox);
 
 	/* Remove any previous wireless security widgets */
@@ -269,6 +302,7 @@ ws_802_1x_auth_combo_changed (GtkWidget *combo,
 
 	eap_widget = eap_method_get_widget (eap);
 	g_assert (eap_widget);
+	gtk_widget_unparent (eap_widget);
 
 	if (size_group)
 		eap_method_add_to_size_group (eap, size_group);
@@ -276,7 +310,7 @@ ws_802_1x_auth_combo_changed (GtkWidget *combo,
 
 	/* Refocus the EAP method's default widget */
 	if (eap->default_field) {
-		eap_default_widget = glade_xml_get_widget (eap->xml, eap->default_field);
+		eap_default_widget = GTK_WIDGET (gtk_builder_get_object (eap->builder, eap->default_field));
 		if (eap_default_widget)
 			gtk_widget_grab_focus (eap_default_widget);
 	}
@@ -288,7 +322,6 @@ ws_802_1x_auth_combo_changed (GtkWidget *combo,
 
 GtkWidget *
 ws_802_1x_auth_combo_init (WirelessSecurity *sec,
-                           const char *glade_file,
                            const char *combo_name,
                            GCallback auth_combo_changed_cb,
                            NMConnection *connection,
@@ -297,6 +330,7 @@ ws_802_1x_auth_combo_init (WirelessSecurity *sec,
 	GtkWidget *combo;
 	GtkListStore *auth_model;
 	GtkTreeIter iter;
+	EAPMethodSimple *em_md5;
 	EAPMethodTLS *em_tls;
 	EAPMethodLEAP *em_leap;
 	EAPMethodTTLS *em_ttls;
@@ -323,7 +357,24 @@ ws_802_1x_auth_combo_init (WirelessSecurity *sec,
 
 	auth_model = gtk_list_store_new (2, G_TYPE_STRING, eap_method_get_g_type ());
 
-	em_tls = eap_method_tls_new (glade_file, sec, connection, FALSE);
+	if (wired) {
+		em_md5 = eap_method_simple_new (sec,
+		                                connection,
+		                                EAP_METHOD_SIMPLE_TYPE_MD5,
+		                                FALSE,
+		                                is_editor);
+		gtk_list_store_append (auth_model, &iter);
+		gtk_list_store_set (auth_model, &iter,
+			                AUTH_NAME_COLUMN, _("MD5"),
+			                AUTH_METHOD_COLUMN, em_md5,
+			                -1);
+		eap_method_unref (EAP_METHOD (em_md5));
+		if (default_method && (active < 0) && !strcmp (default_method, "md5"))
+			active = item;
+		item++;
+	}
+
+	em_tls = eap_method_tls_new (sec, connection, FALSE);
 	gtk_list_store_append (auth_model, &iter);
 	gtk_list_store_set (auth_model, &iter,
 	                    AUTH_NAME_COLUMN, _("TLS"),
@@ -335,7 +386,7 @@ ws_802_1x_auth_combo_init (WirelessSecurity *sec,
 	item++;
 
 	if (!wired) {
-		em_leap = eap_method_leap_new (glade_file, sec, connection);
+		em_leap = eap_method_leap_new (sec, connection);
 		gtk_list_store_append (auth_model, &iter);
 		gtk_list_store_set (auth_model, &iter,
 		                    AUTH_NAME_COLUMN, _("LEAP"),
@@ -347,7 +398,7 @@ ws_802_1x_auth_combo_init (WirelessSecurity *sec,
 		item++;
 	}
 
-	em_ttls = eap_method_ttls_new (glade_file, sec, connection, is_editor);
+	em_ttls = eap_method_ttls_new (sec, connection, is_editor);
 	gtk_list_store_append (auth_model, &iter);
 	gtk_list_store_set (auth_model, &iter,
 	                    AUTH_NAME_COLUMN, _("Tunneled TLS"),
@@ -358,7 +409,7 @@ ws_802_1x_auth_combo_init (WirelessSecurity *sec,
 		active = item;
 	item++;
 
-	em_peap = eap_method_peap_new (glade_file, sec, connection, is_editor);
+	em_peap = eap_method_peap_new (sec, connection, is_editor);
 	gtk_list_store_append (auth_model, &iter);
 	gtk_list_store_set (auth_model, &iter,
 	                    AUTH_NAME_COLUMN, _("Protected EAP (PEAP)"),
@@ -369,7 +420,7 @@ ws_802_1x_auth_combo_init (WirelessSecurity *sec,
 		active = item;
 	item++;
 
-	combo = glade_xml_get_widget (sec->xml, combo_name);
+	combo = GTK_WIDGET (gtk_builder_get_object (sec->builder, combo_name));
 	g_assert (combo);
 
 	gtk_combo_box_set_model (GTK_COMBO_BOX (combo), GTK_TREE_MODEL (auth_model));
@@ -407,7 +458,7 @@ ws_802_1x_fill_connection (WirelessSecurity *sec,
 	s_8021x = (NMSetting8021x *) nm_setting_802_1x_new ();
 	nm_connection_add_setting (connection, (NMSetting *) s_8021x);
 
-	widget = glade_xml_get_widget (sec->xml, combo_name);
+	widget = GTK_WIDGET (gtk_builder_get_object (sec->builder, combo_name));
 	model = gtk_combo_box_get_model (GTK_COMBO_BOX (widget));
 	gtk_combo_box_get_active_iter (GTK_COMBO_BOX (widget), &iter);
 	gtk_tree_model_get (model, &iter, AUTH_METHOD_COLUMN, &eap, -1);
@@ -431,7 +482,7 @@ ws_802_1x_update_secrets (WirelessSecurity *sec,
 	g_return_if_fail (combo_name != NULL);
 	g_return_if_fail (connection != NULL);
 
-	widget = glade_xml_get_widget (sec->xml, combo_name);
+	widget = GTK_WIDGET (gtk_builder_get_object (sec->builder, combo_name));
 	g_return_if_fail (widget != NULL);
 	model = gtk_combo_box_get_model (GTK_COMBO_BOX (widget));
 
@@ -456,7 +507,7 @@ ws_802_1x_nag_user (WirelessSecurity *sec,
 	EAPMethod *eap = NULL;
 	GtkWidget *widget;	
 
-	widget = glade_xml_get_widget (sec->xml, combo_name);
+	widget = GTK_WIDGET (gtk_builder_get_object (sec->builder, combo_name));
 	model = gtk_combo_box_get_model (GTK_COMBO_BOX (widget));
 	gtk_combo_box_get_active_iter (GTK_COMBO_BOX (widget), &iter);
 	gtk_tree_model_get (model, &iter, AUTH_METHOD_COLUMN, &eap, -1);
